// STRING.HPP
// Goal: Design a string class that looks and feels like a number, in that
//   they can be created, passed, compared, changed, and destroyed easily.

#ifndef __STRING_HPP
#define __STRING_HPP

#include <iostream.h>
#include <string.h>
#include <assert.h>


class string {
protected:
  char *s;
  void dup(char *source);              // copies, doesn't dealloc first
  void kill();                         // dealloc if str != 0

public:
  static int count;
  static int pchar_count;

  string(string &src) { count++; dup(src.s); }
  string(char *src) { count++; dup(src); }
  string(char single) { count++; s=0; fromchar(single); }
  string(int length) { count++; s=0; setlength(length); }
  string() { count++; s=0; }

  ~string() { count--; kill(); }

  int length() { return s? strlen(s) : 0; }
  int isempty() { return !s; }
  int contains(char c) { return (int)strchr(s, c); }

  char& operator[] (int i) { return s[i]; }

  // conversions
  operator char* () { return s; }      // I'm wary of doing this...
  char *pchar() { return s; }
  int toInt();
  long toLong();
  float toFloat();
  double toDouble();

  string &fromchar(char single);
  string &fromint(long val);
  string &fromfloat(float val) { return fromfloat((double)val); }
  string &fromfloat(double val);

  // assignment; allow char* here but no where else
  string& operator=(string &src) { kill(); dup(src.s); return *this; }
  string& operator=(char *src) { kill(); dup(src); return *this; }

  string& setlength(int newlen);       // uninitialized, except for null

  // comparison
  int operator-(string &src) { assert(s && src.s); return strcmp(s, src.s); }

  #define MAKEOP(op) \
  int operator op (string &src) { return *this - src op 0; }
  MAKEOP(==)  MAKEOP(!=)
  MAKEOP(>=)  MAKEOP(>)
  MAKEOP(<=)  MAKEOP(<)
  #undef MAKEOP

  // concatenation
  string operator+(string &tail);
  string& operator+=(string &tail);

  // input/output
  friend istream& operator>> (istream &is, string &obj)
    { return obj.readline(is); }
  friend ostream& operator<< (ostream &is, string &obj)
    { return obj.write(is); }

  istream& readline(istream &is);      // all chars to first CR or LF
  istream& readword(istream &is);      // all chars to first whitespace
  ostream& write(ostream &os);         // writes all stored characters
};

#endif
