/* TEST23.CPP
  Tests and exersizes the tree23 module. The program has two modes,
interactive and automatic.
  In interactive mode, the program first tests simple insertions. It asks
for a key over and over until "quit" is typed; each key is immediately
inserted into a tree and displayed structurally.
  Next (still in interactive mode), the program prompts for N, the number of
nodes to calculate the average value of (nodes/keys) by calculating this
value for all permutations of an sequence of numbers of length N.

  In automatic mode, signaled by the presence of command-line arguments, the
program will do only the permutation calculation. The input sequence is
always of the form 1,2,3,...,N. To allow distributed calculation of this
value the 1st and 2nd command-line arguments are the 1st and 2nd elements
to use. They are removed from the sequence and always appear at the beginning
of the permutations. The value that is printed is the sum of the number of
nodes in all the trees; it is expected that the number of keys will be
calculated by the user as (N*N!).

*/

#error Obsolete, I think...

#include "tree23.hpp"
#include "slist.hpp"
#include <string.h>
#include <stdlib.h>
#include "my_timer.h"


typedef datasnode<int> numnode;

numnode *enumerate(int toWhat)
{
  if (toWhat <= 0)
    return 0;
  else
    return (numnode*)append( enumerate(toWhat-1), new numnode(toWhat) );
}


long double nodesum;
unsigned long tempsum;

int runtest(numnode *data, void*)
{
  tree23 *t = maketree(data->data);
  data = data->Next();

  while (data) {
    t = insert(t, data->data);
    data = data->Next();
  }

  tempsum += tree23::count;

  if (tempsum > 0x40000000) {
    nodesum += tempsum;       // avoid floating-point math
    tempsum = 0;
  }

  destroy(t);
  return 1;
}


int first, second;

int runtest_given2(numnode *data, void*)
{
  tree23 *t = maketree(first);
  t = insert(t, second);

  while (data) {
    t = insert(t, data->data);
    data = data->Next();
  }

  tempsum += tree23::count;

  if (tempsum > 0x40000000) {
    nodesum += tempsum;       // avoid floating-point math
    tempsum = 0;
  }

  destroy(t);
  return 1;
}


long double fact(int i)
{
  long double k=1;
  while (i)
    k *= (long double)(i--);
  return k;
}


// args: 1,2 = first 2 in all permutations
//       3 = number of #s to use (5 -> 1,2,3,4,5)
int main(int argc, char *argv[])
{
  initpool();
  double elapsed;
  char buf[80];
  tree23 *t = 0;

  if (argc < 4) {
    cin >> buf;
    t = 0;

    while (strcmp(buf, "quit")) {
      if (!t)
        t = maketree(atoi(buf));
      else
        t = insert(t, atoi(buf));

      cout << *t << endl;
      t->treeprint(0);
      cout << "Check: " << t->check() << endl;
      cout << "Nodes: " << t->countnodes() << endl;
      cout << "Quick: " << tree23::count << endl;

      int levels = t->levels();
      if (levels <= 3) {
        string type = leveltype(levels, t);
        cout << "Type:   " << type << endl;

        tree23 *mirror = makeleveltree(levels, type);
        cout << "Mirror: " << leveltype(levels, mirror) << endl;
        destroy(mirror);
      }

      cin >> buf;
    }
    destroy(t);

    cout << "Characteristics: level, type; level=-1 to go on\n";
    int level=0, type;
    while (level != -1) {
      cin >> level >> type;
      printcharacteristics(level, type);
    }

    cin >> buf;
  }

  else {
    first = atoi(argv[1]);
    second = atoi(argv[2]);
    strcpy(buf, argv[3]);
  }

  #define max(a,b) ((a)>(b)?(a):(b))
  #define min(a,b) ((a)<(b)?(a):(b))
  while (strcmp(buf, "quit")) {
    nodesum = 0;
    tempsum = 0;

    numnode *list = enumerate(atoi(buf));
    if (argc >= 4) {
      list = (numnode*) remove(list, nth(list, max(first,second)-1));
      list = (numnode*) remove(list, nth(list, min(first,second)-1));
    }

    my_timer_start();
    if (argc < 4)
      permute(list, (snode_predicate)runtest);
    else
      permute(list, (snode_predicate)runtest_given2);
    elapsed = my_timer_seconds();

    destroy(list);

    nodesum += tempsum;

    if (argc < 4) {
      double result = nodesum / (((long double)atoi(buf))
                                  * fact(atoi(buf)));
      cout << atoi(buf) << " keys, sum = " << (double)nodesum
           << ", nodes/keys = " << result
           << "    Time: " << elapsed << " seconds"
           << endl;

      cin >> buf;
    }
    else {
      cout << (double)nodesum << endl;
      strcpy(buf, "quit");
    }
  }

  return 0;
}
